<?php

namespace Modules\Qc\Http\Controllers;

use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Qc\Entities\IacListAttachment;
use Modules\Qc\Entities\IacList;

class IacListController extends Controller
{
    public function uploadAttachment(Request $request)
    {
        try {
            $token = $request->get('attachment_token');
            $file = $request->file('file');
            $file_name = $file->getClientOriginalName();
            $filename = uniqid() . '_' . time() . '.' . $file_name;
            $file->move(public_path('iac_list'), $filename);

            $iac_list_attachment = new IacListAttachment();
            $iac_list_attachment->internal_audit_id = 0;
            $iac_list_attachment->path = $filename;
            $iac_list_attachment->token = $token;
            $iac_list_attachment->save();

            $responseBody = $this->responseBody(true, "IacListController", "uploaded", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "IacListController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function allIacLists()
    {
        try {
            $query = "SELECT
            qc_iac_list.*,
            qc_iac_list_attachments.path,
            pp_customer_order_plans_data.manufacturing_order_no,
            st_products.product_name,
            hr_employee_sites.site
        FROM
            qc_iac_list_attachments
            INNER JOIN
            qc_iac_list
            ON
                qc_iac_list_attachments.internal_audit_id = qc_iac_list.internal_audit_id
            INNER JOIN
            pp_customer_order_plans_data
            ON
                qc_iac_list.customer_order_plan_data_id = pp_customer_order_plans_data.customer_order_plan_data_id
            INNER JOIN
            st_products
            ON
                qc_iac_list.product_primary_id = st_products.product_primary_id
            INNER JOIN
            hr_employee_sites
            ON
                qc_iac_list.factory = hr_employee_sites.site_id
                GROUP BY
            qc_iac_list.internal_audit_id
            ";

            $records = DB::select($query);
            $result = array();
            foreach ($records as $record) {
                array_push($result, [
                    "internal_audit_id" => $record->internal_audit_id,
                    "reference_no" => $record->reference_no,
                    "product_name" => $record->product_name,
                    "manufacturing_order_no" => $record->manufacturing_order_no,
                    "date" => $record->date,
                    "factory" => $record->site,
                    "description" => $record->description,
                    "path" => $record->path,
                ]);
            }
            $responseBody = $this->responseBody(true, "IacListController", "allIacLists", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "IacListController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function saveIacList(Request $request)
    {
        try {
            $ncr = new IacList();
            $ncr->user_id = Auth::user()->id;
            $ncr->reference_no = $request->get('reference_no');
            $ncr->date = $request->get('date');
            $ncr->customer_order_plan_data_id = $request->get('customer_order_plan_data_id');
            $ncr->product_primary_id = $request->get('product_primary_id');
            $ncr->factory = $request->get('factory');
            $ncr->description = $request->get('description');
            if ($ncr->save()) {
                $token = $request->get('attachment_token');
                $attachment = IacListAttachment::where('token', '=', $token)->first();
                if ($attachment) {
                    $attachment->internal_audit_id = $ncr->internal_audit_id;
                    $attachment->update();
                }
            }
            $responseBody = $this->responseBody(true, "IacListController", "saveIacList", $attachment);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "IacListController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function update(Request $request, $id)
    {
        try {
            $ncr =  IacList::find($id);
            $ncr->user_id = Auth::user()->id;
            $ncr->reference_no = $request->get('reference_no');
            $ncr->date = $request->get('date');
            $ncr->customer_order_plan_data_id = $request->get('customer_order_plan_data_id');
            $ncr->product_primary_id = $request->get('product_primary_id');
            $ncr->factory = $request->get('factory');
            $ncr->description = $request->get('description');
            if ($ncr->update()) {
                $token = $request->get('attachment_token');
                $attachments = IacListAttachment::where('token', '=', $token)->get();
                if (count($attachments) > 1) {
                    if ($this->delete($id)) {
                        $attachment = IacListAttachment::where('token', '=', $token)->first();
                        $attachment->internal_audit_id = (int)$id;
                        $attachment->update();
                    }
                }
                $responseBody = $this->responseBody(true, "IacListController", "updated", count($attachments));
            }
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "IacListController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }




    public function delete($id)
    {
        try {
            $attachment = IacListAttachment::where('internal_audit_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            return true;
        } catch (Exception $exception) {
            return false;
        }
    }


    public function deleteRecord($id)
    {
        try {
            $record = IacList::where('internal_audit_id', '=', $id)->get();
            foreach ($record as $data) {
                $data->delete();
            }
            $attachment = IacListAttachment::where('internal_audit_id', '=', $id)->get();
            foreach ($attachment as $data) {
                $data->delete();
            }
            $responseBody = $this->responseBody(true, "IacListController", "deleted", null);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "IacListController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }

    public function getIaclData($id)
    {
        try {
            $query = "SELECT
            qc_iac_list.*,
            qc_iac_list_attachments.token
        FROM
            qc_iac_list_attachments
            INNER JOIN
            qc_iac_list
            ON
                qc_iac_list_attachments.internal_audit_id = qc_iac_list.internal_audit_id
        WHERE
            qc_iac_list.internal_audit_id = {$id}";

            $ncr =  DB::select($query);

            $responseBody = $this->responseBody(true, "IacListController", "getIaclData", $ncr[0]);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "IacListController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }



    /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }
}
