<?php

namespace Modules\Qc\Http\Controllers;

use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Contracts\Support\Renderable;
use Illuminate\Http\Request;
use Illuminate\Routing\Controller;
use Modules\Qc\Entities\SupervisorCheckListData;
use Modules\Qc\Entities\SupervisorCheckListHeader;
use Modules\Qc\Entities\MaterialTypeRemarks;
use Modules\Qc\Entities\MachineRemarks;

class SupervisorCheckListHeaderController extends Controller
{
    public function allRecords()
    {
        try {
            $query = "SELECT
            pp_customer_order_plans_data.manufacturing_order_no,
            st_products.product_name,
            pp_customer_orders.purchase_order,
            qc_supervisor_check_list_headers.date,
            qc_supervisor_check_list_headers.reference_no,
            qc_supervisor_check_list_headers.scl_id,
            hr_att_shifts.shift_name,
            hr_employee_sites.site
        FROM
            st_products
            INNER JOIN
            pp_customer_order_plans_data
            INNER JOIN
            pp_customer_orders
            ON
                pp_customer_order_plans_data.customer_order_plan_data_id = pp_customer_orders.order_id
            INNER JOIN
            qc_supervisor_check_list_headers
            ON
                st_products.product_primary_id = qc_supervisor_check_list_headers.product_primary_id AND
                pp_customer_order_plans_data.customer_order_plan_data_id = qc_supervisor_check_list_headers.customer_order_plan_data_id
            INNER JOIN
            hr_att_shifts
            ON
                qc_supervisor_check_list_headers.shift_id = hr_att_shifts.shift_id
            INNER JOIN
            hr_employee_sites
            ON
                qc_supervisor_check_list_headers.factory = hr_employee_sites.site_id
        GROUP BY
            qc_supervisor_check_list_headers.scl_id";

            $records = DB::select($query);
            $result = array();
            foreach ($records as $record) {
                array_push($result, [
                    "scl_id" => $record->scl_id,
                    "reference_no" => $record->reference_no,
                    "purchase_order" => $record->purchase_order,
                    "product_name" => $record->product_name,
                    "manufacturing_order_no" => $record->manufacturing_order_no,
                    "factory" => $record->site,
                    "shift" => $record->shift_name,
                    "date" => $record->date,
                ]);
            }
            $responseBody = $this->responseBody(true, "PackingLineQualityRecordHeaderController", "allRecords", $result);
        } catch (Exception $exception) {
            $responseBody = $this->responseBody(false, "PackingLineQualityRecordHeaderController", "error", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }


    public function deleteRecord($id)
    {
        try {
            if ($this->delete($id)) {
                $rec_data = SupervisorCheckListHeader::where('scl_id', '=', $id)->get();
                foreach ($rec_data as $data) {
                    $data->delete();
                }
                return true;
            }
        } catch (Exception $exception) {
            return false;
        }
    }

    public function delete($id)
    {
        try {
            $rec_data = SupervisorCheckListData::where('scl_id', '=', $id)->get();
            foreach ($rec_data as $data) {
                $data->delete();
            }
            $rec_data = MaterialTypeRemarks::where('scl_id', '=', $id)->get();
            foreach ($rec_data as $data) {
                $data->delete();
            }
            $rec_data = MachineRemarks::where('scl_id', '=', $id)->get();
            foreach ($rec_data as $data) {
                $data->delete();
            }
            return true;
        } catch (Exception $exception) {
            return false;
        }
    }


    public function supervisorCheckListReport($id)
    {
        try {
            $query1 = "SELECT
            qc_packing_line_quality_record_headers.*,
            pp_customer_order_plans_data.manufacturing_order_no,
            st_products.product_name,
            pp_customer_orders.purchase_order
        FROM
            qc_packing_line_quality_record_headers
            INNER JOIN
            st_products
            ON
                qc_packing_line_quality_record_headers.product_primary_id = st_products.product_primary_id
            INNER JOIN
            pp_customer_order_plans_data
            ON
                qc_packing_line_quality_record_headers.customer_order_plan_data_id = pp_customer_order_plans_data.customer_order_plan_data_id
            INNER JOIN
            pp_customer_orders
            ON
                qc_packing_line_quality_record_headers.customer_order_plan_data_id = pp_customer_orders.order_id
        WHERE
	        qc_packing_line_quality_record_headers.plqr_id = {$id}";

            $records = DB::select($query1);

            $header = [
                "order_no" => "",
                "product" => "",
                "factory_order_number" => "",
                "trans_date" => "",
            ];


            if ($records) {
                $header["order_no"] = $records[0]->purchase_order;
                $header["product"] = $records[0]->product_name;
                $header["factory_order_number"] = $records[0]->manufacturing_order_no;
                $header["trans_date"] = $records[0]->trans_date;
            }

            $query2 = "SELECT
            qc_packing_line_quality_record_data.*,
            hr_employees.name_withinitial
        FROM
            hr_employees
            INNER JOIN
            qc_packing_line_quality_record_data
            ON
                hr_employees.employee_id = qc_packing_line_quality_record_data.checked_by
                WHERE
                qc_packing_line_quality_record_data.plqr_id = {$id}";

            $data = DB::select($query2);

            $records[0]->request_data = $data;


            $responseBody = $this->responseBody(true, "DamageCheckingRecordsHeaderController", "damageCheckingReport", ["header" => $header, "body" => ["data" => $data]]);
        } catch (\Exception $exception) {
            $responseBody = $this->responseBody(false, "DamageCheckingRecordsHeaderController", "damageCheckingReport", $exception);
        }

        return response()->json(["data" => $responseBody]);
    }



     /**
     * responseBody
     * This is used to create response.
     * @param success This is the paramter require boolean
     * @param name This is the paramter require ui table name
     * @param message This is the paramter require message content
     * @param result This is the paramter require result as some of data to return client
     * @return Json This returns as response.
     */
    private function responseBody($success, $name, $message, $result)
    {
        $body = [
            "success" => $success,
            "message" => $message,
            "name" => $name,
            "result" => $result
        ];
        return $body;
    }

}
